﻿'
' Demonstrates 1-wire usage with the DS1822. Also, demonstrates some basic UD 
' functionality by setting a DAC and reading an AIN.
'
' support@labjack.com
'

Option Explicit On

' Import the UD .NET wrapper object. The LJUDDotNet(.dll) assembly
' needs to be referenced in your project settings. This dll is installed
' by the LabJack Windows software installer.
Imports LabJack.LabJackUD


Module Module1

    Const DEBUG As Boolean = False ' Turns on/off displaying the 1-wire command/response arrays

    Sub showErrorMessage(ByVal err As LabJackUDException)
        Console.WriteLine("Function returned LabJackUD Error #" & _
            Str$(err.LJUDError) & _
            "  " & _
            err.ToString)
    End Sub

    Sub setExtendedChecksum(ByRef bytes() As Byte)
        Dim cs As UInt32
        Dim tmp As UInt32

        ' Extended Checksum16
        ' Sums bytes 6 to n-1 to a unsigned 2 byte value
        cs = 0
        For i As Integer = 6 To bytes.Length - 1
            cs = cs + bytes(i)
        Next
        bytes(4) = cs And 255
        bytes(5) = cs >> 8

        ' Extended Checksum8 
        ' Sums bytes 1 to 5. Sums quotient and remainder of 256 division. Again,
        ' sums quotient and remainder of 256 division.
        cs = 0
        For i As Integer = 1 To 5
            cs = cs + bytes(i)
        Next
        tmp = cs \ 256
        cs = (cs - 256 * tmp) + tmp
        tmp = cs \ 256
        bytes(0) = CByte((cs - 256 * tmp) + tmp)
    End Sub

    ' Parameters are from the 1-Wire low-level function command/response bytes.
    ' Refer here for documentation:
    '   https://labjack.com/support/datasheets/u3/low-level-function-reference/1-wire
    ' Notes:
    '   txBytes needs to be at least numTX in size
    '   data needs to be at least numRX in size. data is basically rxBytes
    '   numTX and txBytes max size is 40.
    '   data and rxBytes max size is 48.
    Sub oneWire(ByVal handle As Integer, _
                ByVal options As Byte, _
                ByVal sensePin As Byte, _
                ByVal dpuPin As Byte, _
                ByVal romFunction As Byte, _
                ByVal rom() As Byte, _
                ByVal numTX As Byte, _
                ByVal numRX As Byte, _
                ByVal txBytes() As Byte, _
                ByRef errorCode As Byte, _
                ByRef warnings As Byte, _
                ByRef data() As Byte)

        Dim sndSize As Double = 64.0
        Dim rcvSize As Double = 64.0
        Dim snd(63) As Byte
        Dim rcv(63) As Byte

        If rom.Length < 8 Then
            Throw New Exception("oneWire: rom array needs 10 elements.")
        End If
        If snd.Length < numTX Then
            Throw New Exception("oneWire: snd array needs at least numTX elements.")
        End If
        If data.Length < numRX Then
            Throw New Exception("oneWire: data array needs at least numRX elements.")
        End If
        If numTX > 40 Then
            Throw New Exception("oneWire: numTX is greater then max. of 40.")
        End If
        If numTX > 40 Then
            Throw New Exception("oneWire: numRX is greater then max. of 48.")
        End If

        ' Build command
        snd(1) = &HF8
        snd(2) = &H1D
        snd(3) = &H3C

        snd(6) = options
        snd(7) = 0 ' Reserved
        snd(8) = sensePin
        snd(9) = dpuPin
        snd(10) = 0 ' reserved
        snd(11) = romFunction
        Array.Copy(rom, 0, snd, 12, 8)
        snd(20) = 0 ' reserved
        snd(21) = numTX
        snd(22) = 0 ' reserved
        snd(23) = numRX
        If numTX > 0 Then
            Array.Copy(txBytes, 0, snd, 24, numTX)
        End If
        setExtendedChecksum(snd)

        If DEBUG Then
            For i As Integer = 0 To sndSize - 1
                Console.Write(Hex(snd(i)) & ", ")
            Next
            Console.WriteLine("")
        End If

        ' eGetPtr and raw I/O IOTypes doc:
        ' https://labjack.com/support/datasheets/u3/high-level-driver/function-reference/eput-eget
        ' https://labjack.com/support/datasheets/u3/high-level-driver/example-pseudocode/raw-input-output

        ' Send command
        LJUD.eGetPtr(handle, LJUD.IO.RAW_OUT, 0, sndSize, snd)

        ' Get response
        LJUD.eGetPtr(handle, LJUD.IO.RAW_IN, 0, rcvSize, rcv)

        If DEBUG Then
            For i As Integer = 0 To rcvSize - 1
                Console.Write(Str$(rcv(i)) & ", ")
            Next
            Console.WriteLine("")
        End If

        If rcv(1) <> &HF8 Then
            Throw New Exception("oneWire: Incorrect Command in response.")

        End If
        If rcv(2) <> &H1D Then
            Throw New Exception("oneWire: Incorrect Number of Data Words in response.")
        End If
        If rcv(3) <> &H3C Then
            Throw New Exception("oneWire: Incorrect Extended Command Number in response.")
        End If

        errorCode = rcv(6)
        warnings = rcv(9)
        If numRX > 0 Then
            Array.Copy(rcv, 16, data, 0, numRX)
        End If
    End Sub

    Sub Main()
        Dim handle As Integer
        Dim dacChannel As Integer
        Dim ainChannel As Integer
        Dim voltage As Double

        ' 1-wire related
        Dim options As Byte
        Dim sensePin As Byte
        Dim dpuPin As Byte
        Dim romFunction As Byte
        Dim rom(10) As Byte
        Dim numTX As Byte
        Dim numRX As Byte
        Dim txBytes() As Byte
        Dim errorCode As Byte
        Dim warnings As Byte
        Dim data() As Byte
        Dim temperatureC As Double

        Try
            ' Open first found device
            ' OpenLabJack doc.: https://labjack.com/support/datasheets/u3/high-level-driver/Function-reference/openlabjack
            LJUD.OpenLabJack(LJUD.DEVICE.U3, LJUD.CONNECTION.USB, "0", True, handle)


            ' Start by using the pin_configuration_reset IOType so that all
            ' pin assignments are in the factory default condition.
            LJUD.ePut(handle, LJUD.IO.PIN_CONFIGURATION_RESET, 0, 0, 0)

            ' Set DAC1 to 3.3 V
            ' eDAC doc.: https://labjack.com/support/datasheets/u3/high-level-driver/function-reference/edac
            dacChannel = 1
            voltage = 3.3
            LJUD.eDAC(handle, dacChannel, voltage, 0, 0, 0)
            Console.WriteLine("Set DAC" & Str$(dacChannel) & " = " & Str$(voltage) & " V")

            ' Read AIN1 voltage (single-ended)
            ' eAIN doc.: https://labjack.com/support/datasheets/u3/high-level-driver/function-reference/eain
            ainChannel = 1
            voltage = 0.0
            LJUD.eAIN(handle, ainChannel, 31, voltage, 0, 0, 0, 0)
            Console.WriteLine("Read AIN" & Str$(ainChannel) & " = " & Str$(voltage) & " V")
            Console.WriteLine("")


            ' Set DAC1 to 3.3 V and read AIN1 voltage (single-ended) with
            ' Add/Go/Get functions.
            ' Does the same as the above eDAC and eAIN calls, but minimizes
            ' USB command/responses to the U3.
            ' AddRequest, GoOne and GetResult documented here:
            ' https://labjack.com/support/datasheets/u3/high-level-driver/function-reference/addrequest
            ' https://labjack.com/support/datasheets/u3/high-level-driver/function-reference/goone
            ' https://labjack.com/support/datasheets/u3/high-level-driver/function-reference/getresult
            dacChannel = 1
            voltage = 3.3
            LJUD.AddRequest(handle, LJUD.IO.PUT_DAC, dacChannel, voltage, 0, 0)
            ainChannel = 1
            LJUD.AddRequest(handle, LJUD.IO.GET_AIN, ainChannel, 0, 0, 0)

            'Execute the requests.
            LJUD.GoOne(handle)

            Console.WriteLine("Set DAC" & Str$(dacChannel) & " = " & Str$(voltage) & " V")
            LJUD.GetResult(handle, LJUD.IO.GET_AIN, ainChannel, voltage) ' Just for error checking.

            voltage = 0.0
            LJUD.GetResult(handle, LJUD.IO.GET_AIN, ainChannel, voltage)
            Console.WriteLine("Read AIN" & Str$(ainChannel) & " = " & Str$(voltage) & " V")
            Console.WriteLine("")


            ' DS1822 1-Wire sensor communications on the LabJack's EIO0 line
            ' Change for you sensor and setup.
            sensePin = 8 ' 8 = EIO0

            ' First configuring EIO0 as digital I/O in case it is not already
            ' ePut and  LJ_ioPUT_ANALOG_ENABLE_BIT (LJUD.IO.PUT_ANALOG_ENABLE_BIT in .NET):
            ' https://labjack.com/support/datasheets/u3/high-level-driver/function-reference/eput-eget
            ' https://labjack.com/support/datasheets/u3/high-level-driver/example-pseudocode/configuration
            LJUD.ePut(handle, LJUD.IO.PUT_ANALOG_ENABLE_BIT, sensePin, 0, 0)


            Console.WriteLine("1-Wire:")
            Console.WriteLine("")


            ' Search for first found sensor to get its ROM ID.
            Console.WriteLine("Searching for a sensor and its ROM ID.")
            options = 0
            dpuPin = 0 ' Not enabled by options, so this setting is ignored
            romFunction = &HF0 ' Search function
            ' 64 bit Sensor ROM ID LSB -> MSB
            rom(0) = 0
            rom(1) = 0
            rom(2) = 0
            rom(3) = 0
            rom(4) = 0
            rom(5) = 0
            rom(6) = 0
            rom(7) = 0

            ' Data to write to the sensor
            numTX = 0 ' Not sending bytes
            ReDim txBytes(0)

            numRX = 8 ' Read back 8 bytes
            ReDim data(numRX - 1) ' Bytes for read data (RX)

            errorCode = 0
            warnings = 0

            ' Perform 1-Wire communications
            oneWire(handle, options, sensePin, dpuPin, romFunction, rom, _
                    numTX, numRX, txBytes, errorCode, warnings, data)

            ' Error code of 0 means no error, > 0 means error.
            ' low-level error code list can be found here:
            '   https://labjack.com/support/datasheets/u3/low-level-function-reference/errorcodes
            Console.WriteLine("  Read Errorcode = " & Str$(errorCode))

            ' Warnings:
            '   Bit 0: No Devices Detected
            '   Bit 1: Type 1 interrupt (Not Tested)
            '   Bit 2: Type 2 interrupt (Not Supported)
            Console.WriteLine("  Read Warnings = " & Str$(warnings))

            Console.Write("  Read Data (ROM ID) = [")
            For i As Integer = 0 To data.Length - 1
                Console.Write(Str$(data(i)) & ", ")
            Next
            Console.WriteLine("]")
            Console.WriteLine("")


            ' Command sensor to perform temperature conversion
            Console.WriteLine("Command sensor to perform temperature conversion")
            options = 0
            dpuPin = 0 ' Not enabled by options, so this setting is ignored

            romFunction = &H55 ' Match function

            ' 64 bit Sensor ROM ID LSB -> MSB
            ' Using the ROM ID from the previous search response (data)
            rom(0) = data(0)
            rom(1) = data(1)
            rom(2) = data(2)
            rom(3) = data(3)
            rom(4) = data(4)
            rom(5) = data(5)
            rom(6) = data(6)
            rom(7) = data(7)

            ' Data to write to the sensor
            ReDim txBytes(0)
            txBytes(0) = &H44 ' Function command 0x44 for convert T
            numTX = txBytes.Length

            numRX = 8 ' Read back 8 bytes
            ReDim data(numRX - 1) ' Bytes for read data (RX)

            errorCode = 0
            warnings = 0

            ' Perform 1-Wire communications
            oneWire(handle, options, sensePin, dpuPin, romFunction, rom, _
                    numTX, numRX, txBytes, errorCode, warnings, data)

            Console.WriteLine("  Read Errorcode = " & Str$(errorCode))
            Console.WriteLine("  Read Warnings = " & Str$(warnings))

            Console.Write("  Read Data = [")
            For i As Integer = 0 To data.Length - 1
                Console.Write(Str$(data(i)) & ", ")
            Next
            Console.WriteLine("]")
            Console.WriteLine("")


            ' Read temperature from sensor
            Console.WriteLine("Read temperature from sensor")

            ' Data to write to the sensor
            ReDim txBytes(0)
            txBytes(0) = &HBE ' Function command 0xBE to read temperature
            numTX = txBytes.Length

            numRX = 2 ' Read back 8 bytes
            ReDim data(numRX - 1) ' Bytes for read data (RX)

            ' Perform 1-Wire communications
            oneWire(handle, options, sensePin, dpuPin, romFunction, rom, _
                    numTX, numRX, txBytes, errorCode, warnings, data)

            Console.WriteLine("  Read Errorcode = " & Str$(errorCode))
            Console.WriteLine("  Read Warnings = " & Str$(warnings))

            Console.Write("  Read Data = [")
            For i As Integer = 0 To data.Length - 1
                Console.Write(Str$(data(i)) & ", ")
            Next
            Console.WriteLine("]")
            temperatureC = (CInt(data(0)) + (CInt(data(1)) << 8)) * 0.0625
            Console.WriteLine("  Temperature Celsius From Data = " & Str$(temperatureC) & " C")
        Catch ljex As LabJackUDException
            showErrorMessage(ljex)
            Console.WriteLine(ljex.StackTrace())
        Catch ex As Exception
            Console.WriteLine(ex.ToString())
            Console.WriteLine(ex.StackTrace())
        End Try

        Console.WriteLine("")
        Console.WriteLine("Done. Press Enter to exit.")
        Console.ReadLine()
    End Sub

End Module
